package ch.phoenix.oss.quarkus.commons.digest;

import java.nio.charset.Charset;
import java.security.MessageDigest;

/**
 * Service for creating and using MessageDigest instances.
 * <p>
 * To configure the default algorithm, see {@link MessageDigestConfiguration}.
 */
public interface MessageDigestService {

    /**
     * Get a new MessageDigest instance using the configured algorithm.
     *
     * @return The MessageDigest instance
     */
    MessageDigest getInstance();

    /**
     * Get a new MessageDigest instance using the specified algorithm.
     *
     * @param algorithm the algorithm to use
     * @return The MessageDigest instance
     */
    MessageDigest getInstance(String algorithm);

    /**
     * Digests the input using the default charset (UTF-8) and
     * the configured algorithm.
     *
     * @param input the input to digest
     * @return the digest
     */
    byte[] digest(String input);

    /**
     * Digests the input using the specified charset and
     * the configured algorithm.
     *
     * @param input   the input to digest
     * @param charset the charset to use
     * @return the digest
     */
    byte[] digest(String input, Charset charset);

    /**
     * Digests the input using the specified charset and algorithm.
     *
     * @param input     the input to digest
     * @param charset   the charset to use
     * @param algorithm the algorithm to use
     * @return the digest
     */
    byte[] digest(String input, Charset charset, String algorithm);

    /**
     * Digests the input using the specified charset and message digest.
     *
     * @param input         the input to digest
     * @param charset       the charset to use
     * @param messageDigest the message digest to use
     * @return the digest
     */
    byte[] digest(String input, Charset charset, MessageDigest messageDigest);
}
